@@ -36,6 +36,10 @@ HashMgr::HashMgr(const char * tpath, const char * apath)
   flag_mode = FLAG_CHAR;
   complexprefixes = 0;
   utf8 = 0;
+  langnum = 0;
+  lang = NULL;
+  enc = NULL;
+  csconv = 0;
   ignorechars = NULL;
   ignorechars_utf16 = NULL;
   ignorechars_utf16_len = 0;
@@ -43,6 +47,7 @@ HashMgr::HashMgr(const char * tpath, const char * apath)
   aliasf = NULL;
   numaliasm = 0;
   aliasm = NULL;
+  forbiddenword = FLAG_NULL; // forbidden word signing flag
   load_config(apath);  
   int ec = load_tables(tpath);
   if (ec) {
@@ -65,7 +70,7 @@ HashMgr::~HashMgr()
       struct hentry * pt = &tableptr[i];
       struct hentry * nt = NULL;
       if (pt) {
-        if (pt->astr && !aliasf) free(pt->astr);
+        if (pt->astr && (!aliasf || TESTAFF(pt->astr, ONLYUPCASEFLAG, pt->alen))) free(pt->astr);
         if (pt->word) free(pt->word);
 #ifdef HUNSPELL_EXPERIMENTAL
         if (pt->description && !aliasm) free(pt->description);
@@ -74,7 +79,7 @@ HashMgr::~HashMgr()
       }
       while(pt) {
         nt = pt->next;
-        if (pt->astr && !aliasf) free(pt->astr);
+        if (pt->astr && (!aliasf || TESTAFF(pt->astr, ONLYUPCASEFLAG, pt->alen))) free(pt->astr);
         if (pt->word) free(pt->word);
 #ifdef HUNSPELL_EXPERIMENTAL
         if (pt->description && !aliasm) free(pt->description);
@@ -101,6 +106,15 @@ HashMgr::~HashMgr()
     free(aliasm);
     aliasm = NULL;
   }  
+
+#ifndef OPENOFFICEORG
+#ifndef MOZILLA_CLIENT
+  if (utf8) free_utf_tbl();
+#endif
+#endif
+
+  if (enc) free(enc);
+  if (lang) free(lang);
   
   if (ignorechars) free(ignorechars);
   if (ignorechars_utf16) free(ignorechars_utf16);
@@ -123,9 +137,15 @@ struct hentry * HashMgr::lookup(const char *word) const
 
 // add a word to the hash table (private)
 
-int HashMgr::add_word(const char * word, int wl, unsigned short * aff, int al, const char * desc)
+int HashMgr::add_word(const char * word, int wl, unsigned short * aff,
+    int al, const char *
+#ifdef HUNSPELL_EXPERIMENTAL
+desc
+#endif
+, bool onlyupcase)
 {
     char * st = mystrdup(word);
+    bool upcasehomonym = false;
     if (wl && !st) return 1;
     if (ignorechars != NULL) {
       if (utf8) {
@@ -159,7 +179,11 @@ int HashMgr::add_word(const char * word, int wl, unsigned short * aff, int al, c
 #endif
     } else {
        struct hentry* hp = (struct hentry *) malloc (sizeof(struct hentry));
-       if (!hp) return 1;
+       if (!hp)
+       {
+           if (st) free(st);
+           return 1;
+       }
        hp->wlen = (short) wl;
        hp->alen = (short) al;
        hp->word = st;
@@ -171,22 +195,98 @@ int HashMgr::add_word(const char * word, int wl, unsigned short * aff, int al, c
             hp->description = (desc) ? get_aliasm(atoi(desc)) : mystrdup(desc);
        } else {
             hp->description = mystrdup(desc);
-            if (desc && !hp->description) return 1;
+            if (desc && !hp->description)
+            {
+                free(hp->word);
+                free(hp->astr);
+                free(hp);
+                return 1;
+            }
             if (dp->description && complexprefixes) {
                 if (utf8) reverseword_utf(hp->description); else reverseword(hp->description);
             }
        }
 #endif
        while (dp->next != NULL) {
-         if ((!dp->next_homonym) && (strcmp(hp->word, dp->word) == 0)) dp->next_homonym = hp;
+         if ((!dp->next_homonym) && (strcmp(hp->word, dp->word) == 0)) {
+    	    // remove hidden onlyupcase homonym
+            if (!onlyupcase) {
+		if ((dp->astr) && TESTAFF(dp->astr, ONLYUPCASEFLAG, dp->alen)) {
+		    free(dp->astr);
+		    dp->astr = hp->astr;
+		    dp->alen = hp->alen;
+		    free(hp->word);
+		    free(hp);
+		    return 0;
+		} else {
+    		    dp->next_homonym = hp;
+    		}
+            } else {
+        	upcasehomonym = true;
+            }
+         }
          dp=dp->next;
        }
-       if ((!dp->next_homonym) && (strcmp(hp->word, dp->word) == 0)) dp->next_homonym = hp;
-       dp->next = hp;
+       if (strcmp(hp->word, dp->word) == 0) {
+    	    // remove hidden onlyupcase homonym
+            if (!onlyupcase) {
+		if ((dp->astr) && TESTAFF(dp->astr, ONLYUPCASEFLAG, dp->alen)) {
+		    free(dp->astr);
+		    dp->astr = hp->astr;
+		    dp->alen = hp->alen;
+		    free(hp->word);
+		    free(hp);
+		    return 0;
+		} else {
+    		    dp->next_homonym = hp;
+    		}
+            } else {
+        	upcasehomonym = true;
+            }
+       }
+       if (!upcasehomonym) {
+    	    dp->next = hp;
+       } else {
+    	    // remove hidden onlyupcase homonym
+    	    free(hp->word);
+    	    if (hp->astr) free(hp->astr);
+    	    free(hp);
+       }
     }
     return 0;
 }     
 
+int HashMgr::add_hidden_capitalized_word(char * word, int wl,
+    unsigned short * flags, int al, char * dp)
+{
+    // add inner capitalized forms to handle the following allcap forms:
+    // Mixed caps: OpenOffice.org -> OPENOFFICE.ORG
+    // Allcaps with suffixes: CIA's -> CIA'S
+    int captype = utf8 ? get_captype_utf8(word, wl, langnum) : get_captype(word, wl, csconv);
+    if (((captype == HUHCAP) || (captype == HUHINITCAP) ||
+      ((captype == ALLCAP) && (flags != NULL))) &&
+      !((flags != NULL) && TESTAFF(flags, forbiddenword, al))) {
+          unsigned short * flags2 = (unsigned short *) malloc (sizeof(unsigned short) * (al+1));
+	  if (!flags2) return 1;
+          if (al) memcpy(flags2, flags, al * sizeof(unsigned short));
+          flags2[al] = ONLYUPCASEFLAG;
+          if (utf8) {
+              char st[MAXDELEN];
+              w_char w[MAXDELEN];
+              int wlen = u8_u16(w, MAXDELEN, word);
+              mkallsmall_utf(w, wlen, langnum);
+              mkallcap_utf(w, 1, langnum);
+              u16_u8(st, MAXDELEN, w, wlen);
+              return add_word(st,wl,flags2,al+1,dp, true);
+           } else {
+               mkallsmall(word, csconv);
+               mkinitcap(word, csconv);
+               return add_word(word,wl,flags2,al+1,dp, true);
+           }
+    }
+    return 0;
+}
+
 // add a custom dic. word to the hash table (public)
 int HashMgr::put_word(const char * word, int wl, char * aff)
 {
@@ -198,19 +298,26 @@ int HashMgr::put_word(const char * word, int wl, char * aff)
     } else {
         flags = NULL;
     }
-    add_word(word, wl, flags, al, NULL);
-    return 0;
+    add_word(word, wl, flags, al, NULL, false);
+    return add_hidden_capitalized_word((char *) word, wl, flags, al, NULL);
 }
 
 int HashMgr::put_word_pattern(const char * word, int wl, const char * pattern)
 {
-    unsigned short * flags;
     struct hentry * dp = lookup(pattern);
-    if (!dp || !dp->astr) return 1;
-    flags = (unsigned short *) malloc (dp->alen * sizeof(short));
-    memcpy((void *) flags, (void *) dp->astr, dp->alen * sizeof(short));
-    add_word(word, wl, flags, dp->alen, NULL);
-    return 0;
+    if (dp && dp->astr) {
+	if (aliasf) {
+	    add_word(word, wl, dp->astr, dp->alen, NULL, false);	
+	} else {
+    	    unsigned short * flags = (unsigned short *) malloc (dp->alen * sizeof(short));
+	    if (flags) {
+		memcpy((void *) flags, (void *) dp->astr, dp->alen * sizeof(short));
+		add_word(word, wl, flags, dp->alen, NULL, false);
+	    } else return 1;
+	}
+    	return add_hidden_capitalized_word((char *) word, wl, dp->astr, dp->alen, NULL);
+    }
+    return 1;
 }
 
 // walk the hash table entry by entry - null at end
@@ -253,24 +360,34 @@ int HashMgr::load_tables(const char * tpath)
 
   // first read the first line of file to get hash table size */
   char ts[MAXDELEN];
-  if (! fgets(ts, MAXDELEN-1,rawdict)) return 2;
+  if (! fgets(ts, MAXDELEN-1,rawdict)) {
+    HUNSPELL_WARNING(stderr, "error: empty dic file\n");
+    fclose(rawdict);
+    return 2;
+  }
   mychomp(ts);
   
   /* remove byte order mark */
-  if (strncmp(ts,"﻿",3) == 0) {
+  if (strncmp(ts,"\xEF\xBB\xBF",3) == 0) {
     memmove(ts, ts+3, strlen(ts+3)+1);
     HUNSPELL_WARNING(stderr, "warning: dic file begins with byte order mark: possible incompatibility with old Hunspell versions\n");
   }
   
   if ((*ts < '1') || (*ts > '9')) HUNSPELL_WARNING(stderr, "error - missing word count in dictionary file\n");
   tablesize = atoi(ts);
-  if (!tablesize) return 4; 
+  if (!tablesize) {
+    fclose(rawdict);
+    return 4;
+  }
   tablesize = tablesize + 5 + USERWORD;
   if ((tablesize %2) == 0) tablesize++;
 
   // allocate the hash table
   tableptr = (struct hentry *) calloc(tablesize, sizeof(struct hentry));
-  if (! tableptr) return 3;
+  if (! tableptr) {
+    fclose(rawdict);
+    return 3;
+  }
   for (int i=0; i<tablesize; i++) tableptr[i].word = NULL;
 
   // loop through all words on much list and add to hash
@@ -322,10 +439,12 @@ int HashMgr::load_tables(const char * tpath)
     }
 
     wl = strlen(ts);
-
-    // add the word and its index
-    if (add_word(ts,wl,flags,al,dp)) return 5;
-
+    // add the word and its index plus its capitalized form optionally
+    if (add_word(ts,wl,flags,al,dp, false) ||
+	add_hidden_capitalized_word(ts, wl, flags, al, dp)) {
+	fclose(rawdict);
+	return 5;
+    }
   }
  
   fclose(rawdict);
@@ -354,8 +473,9 @@ int HashMgr::decode_flags(unsigned short ** result, char * flags) {
       case FLAG_LONG: { // two-character flags (1x2yZz -> 1x 2y Zz)
         len = strlen(flags);
         if (len%2 == 1) HUNSPELL_WARNING(stderr, "error: length of FLAG_LONG flagvector is odd: %s\n", flags);
-        len = len/2;
+        len /= 2;
         *result = (unsigned short *) malloc(len * sizeof(short));
+        if (!*result) return -1;
         for (int i = 0; i < len; i++) {
             (*result)[i] = (((unsigned short) flags[i * 2]) << 8) + (unsigned short) flags[i * 2 + 1]; 
         }
@@ -370,6 +490,7 @@ int HashMgr::decode_flags(unsigned short ** result, char * flags) {
           if (*p == ',') len++;
         }
         *result = (unsigned short *) malloc(len * sizeof(short));
+        if (!*result) return -1;
         dest = *result;
         for (p = flags; *p; p++) {
           if (*p == ',') {
@@ -387,6 +508,7 @@ int HashMgr::decode_flags(unsigned short ** result, char * flags) {
         w_char w[MAXDELEN/2];
         len = u8_u16(w, MAXDELEN/2, flags);
         *result = (unsigned short *) malloc(len * sizeof(short));
+        if (!*result) return -1;
         memcpy(*result, w, len * sizeof(short));
         break;
       }
@@ -394,6 +516,7 @@ int HashMgr::decode_flags(unsigned short ** result, char * flags) {
         unsigned short * dest;
         len = strlen(flags);
         *result = (unsigned short *) malloc(len * sizeof(short));
+        if (!*result) return -1;
         dest = *result;
         for (unsigned char * p = (unsigned char *) flags; *p; p++) {
           *dest = (unsigned short) *p;
@@ -466,7 +589,7 @@ int  HashMgr::load_config(const char * affpath)
        /* remove byte order mark */
        if (firstline) {
          firstline = 0;
-         if (strncmp(line,"﻿",3) == 0) memmove(line, line+3, strlen(line+3)+1);
+         if (strncmp(line,"\xEF\xBB\xBF",3) == 0) memmove(line, line+3, strlen(line+3)+1);
        }
 
         /* parse in the try string */
@@ -481,7 +604,36 @@ int  HashMgr::load_config(const char * affpath)
                 HUNSPELL_WARNING(stderr, "error: FLAG need `num', `long' or `UTF-8' parameter: %s\n", line);
             }
         }
-        if ((strncmp(line,"SET",3) == 0) && isspace(line[3]) && strstr(line, "UTF-8")) utf8 = 1;
+        if (strncmp(line,"FORBIDDENWORD",13) == 0) {
+          char * st = NULL;
+          if (parse_string(line, &st, "FORBIDDENWORD")) {
+             fclose(afflst);
+             return 1;
+          }
+          forbiddenword = decode_flag(st);
+          free(st);
+        }
+        if (strncmp(line, "SET", 3) == 0) {
+    	  if (parse_string(line, &enc, "SET")) {
+             fclose(afflst);
+             return 1;
+          }    	    
+    	  if (strcmp(enc, "UTF-8") == 0) {
+    	    utf8 = 1;
+#ifndef OPENOFFICEORG
+#ifndef MOZILLA_CLIENT
+    	    initialize_utf_tbl();
+#endif
+#endif
+    	  } else csconv = get_current_cs(enc);
+    	}
+        if (strncmp(line, "LANG", 4) == 0) {    	
+    	  if (parse_string(line, &lang, "LANG")) {
+             fclose(afflst);
+             return 1;
+          }    	    
+    	  langnum = get_lang_num(lang);
+    	}
 
        /* parse in the ignored characters (for example, Arabic optional diacritics characters */
        if (strncmp(line,"IGNORE",6) == 0) {
@@ -509,6 +661,7 @@ int  HashMgr::load_config(const char * affpath)
         if (strncmp(line,"COMPLEXPREFIXES",15) == 0) complexprefixes = 1;
         if (((strncmp(line,"SFX",3) == 0) || (strncmp(line,"PFX",3) == 0)) && isspace(line[3])) break;
     }
+    if (csconv == NULL) csconv = get_current_cs("ISO8859-1");
     fclose(afflst);
     return 0;
 }
