@@ -56,7 +56,7 @@ SuggestMgr::SuggestMgr(const char * tryme, int maxn,
         w_char t[MAXSWL];    
         ctryl = u8_u16(t, MAXSWL, tryme);
         ctry_utf = (w_char *) malloc(ctryl * sizeof(w_char));
-        memcpy(ctry_utf, t, ctryl * sizeof(w_char));
+        if (ctry_utf) memcpy(ctry_utf, t, ctryl * sizeof(w_char));
     } else {
         ctry = mystrdup(tryme);
         ctryl = strlen(ctry);
@@ -449,7 +449,6 @@ int SuggestMgr::doubletwochars(char** wlst, const char * word, int ns, int cpdsu
 // perhaps we doubled two characters (pattern aba -> ababa, for example vacation -> vacacation)
 int SuggestMgr::doubletwochars_utf(char ** wlst, const w_char * word, int wl, int ns, int cpdsuggest)
 {
-  w_char        tmpc;
   w_char        candidate_utf[MAXSWL];
   char          candidate[MAXSWUTF8L];
   int state=0;
@@ -567,7 +566,6 @@ int SuggestMgr::forgotchar(char ** wlst, const char * word, int ns, int cpdsugge
    char candidate[MAXSWUTF8L];
    const char * p;
    char *       q;
-   int cwrd;
    time_t timelimit = time(NULL);
    int timer = MINTIMER;
    int wl = strlen(word);
@@ -717,7 +715,7 @@ int SuggestMgr::swapchar_utf(char ** wlst, const w_char * word, int wl, int ns,
       *p = p[1];
       p[1] = tmpc;
       u16_u8(candidate, MAXSWUTF8L, candidate_utf, wl);
-      ns = testsug(wlst, candidate, wl, ns, cpdsuggest, NULL, NULL);
+      ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
       if (ns == -1) return -1;
       p[1] = *p;
       *p = tmpc;
@@ -768,6 +766,7 @@ int SuggestMgr::longswapchar_utf(char ** wlst, const w_char * word, int wl, int
          tmpc = *p;
          *p = *q;
          *q = tmpc;
+         u16_u8(candidate, MAXSWUTF8L, candidate_utf, wl);
          ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
          if (ns == -1) return -1;
          *q = *p;
@@ -795,7 +794,7 @@ int SuggestMgr::movechar(char ** wlst, const char * word, int ns, int cpdsuggest
       *(q-1) = *q;
       *q = tmpc;
       if ((q-p) < 2) continue; // omit swap char
-      ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
+      ns = testsug(wlst, candidate, wl, ns, cpdsuggest, NULL, NULL);
       if (ns == -1) return -1;
     }
     strcpy(candidate, word);
@@ -806,7 +805,7 @@ int SuggestMgr::movechar(char ** wlst, const char * word, int ns, int cpdsuggest
       *(q+1) = *q;
       *q = tmpc;
       if ((p-q) < 2) continue; // omit swap char
-      ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
+      ns = testsug(wlst, candidate, wl, ns, cpdsuggest, NULL, NULL);
       if (ns == -1) return -1;
     }
     strcpy(candidate, word);
@@ -831,7 +830,7 @@ int SuggestMgr::movechar_utf(char ** wlst, const w_char * word, int wl, int ns,
          *q = tmpc;
          if ((q-p) < 2) continue; // omit swap char
          u16_u8(candidate, MAXSWUTF8L, candidate_utf, wl);
-         ns = testsug(wlst, candidate, wl, ns, cpdsuggest, NULL, NULL);
+         ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
          if (ns == -1) return -1;
      }
      memcpy (candidate_utf, word, wl * sizeof(w_char));
@@ -843,7 +842,7 @@ int SuggestMgr::movechar_utf(char ** wlst, const w_char * word, int wl, int ns,
          *q = tmpc;
          if ((p-q) < 2) continue; // omit swap char
          u16_u8(candidate, MAXSWUTF8L, candidate_utf, wl);
-         ns = testsug(wlst, candidate, wl, ns, cpdsuggest, NULL, NULL);
+         ns = testsug(wlst, candidate, strlen(candidate), ns, cpdsuggest, NULL, NULL);
          if (ns == -1) return -1;
      }
      memcpy (candidate_utf, word, wl * sizeof(w_char));
@@ -900,6 +899,7 @@ int SuggestMgr::ngsuggest(char** wlst, char * w, HashMgr* pHMgr)
   while ((hp = pHMgr->walk_hashtable(col, hp))) {
     if ((hp->astr) && (pAMgr) && 
        (TESTAFF(hp->astr, pAMgr->get_forbiddenword(), hp->alen) ||
+          TESTAFF(hp->astr, ONLYUPCASEFLAG, hp->alen) ||
           TESTAFF(hp->astr, pAMgr->get_nosuggest(), hp->alen) ||
           TESTAFF(hp->astr, pAMgr->get_onlyincompound(), hp->alen))) continue;
     sc = ngram(3, word, hp->word, NGRAM_LONGER_WORSE);
@@ -1089,8 +1089,13 @@ int SuggestMgr::checkword(const char * word, int len, int cpdsuggest, int * time
     if (rv) {
         if ((rv->astr) && (TESTAFF(rv->astr,pAMgr->get_forbiddenword(),rv->alen)
                || TESTAFF(rv->astr,pAMgr->get_nosuggest(),rv->alen))) return 0;
-        if (rv->astr && (TESTAFF(rv->astr,pAMgr->get_pseudoroot(),rv->alen) ||
-            TESTAFF(rv->astr,pAMgr->get_onlyincompound(),rv->alen))) rv = NULL;
+        while (rv) {
+    	    if (rv->astr && (TESTAFF(rv->astr,pAMgr->get_pseudoroot(),rv->alen) ||
+                  TESTAFF(rv->astr, ONLYUPCASEFLAG, rv->alen) ||
+            TESTAFF(rv->astr,pAMgr->get_onlyincompound(),rv->alen))) {
+        	rv = rv->next_homonym;
+    	    } else break;
+    	}
     } else rv = pAMgr->prefix_check(word, len, 0); // only prefix, and prefix + suffix XXX
     
     if (rv) {
@@ -1105,8 +1110,9 @@ int SuggestMgr::checkword(const char * word, int len, int cpdsuggest, int * time
     }
 
     // check forbidden words
-    if ((rv) && (rv->astr) && (TESTAFF(rv->astr,pAMgr->get_forbiddenword(),rv->alen)
-      || TESTAFF(rv->astr,pAMgr->get_nosuggest(),rv->alen) ||
+    if ((rv) && (rv->astr) && (TESTAFF(rv->astr,pAMgr->get_forbiddenword(),rv->alen) ||
+      TESTAFF(rv->astr, ONLYUPCASEFLAG, rv->alen) ||
+      TESTAFF(rv->astr,pAMgr->get_nosuggest(),rv->alen) ||
       TESTAFF(rv->astr,pAMgr->get_onlyincompound(),rv->alen))) return 0;
 
     if (rv) { // XXX obsolote    
@@ -1408,6 +1414,7 @@ char * SuggestMgr::suggest_morph_for_spelling_error(const char * word)
 {
     char * p = NULL;
     char ** wlst = (char **) calloc(maxSug, sizeof(char *));
+    if (!**wlst) return NULL;
     // we will use only the first suggestion
     for (int i = 0; i < maxSug - 1; i++) wlst[i] = "";
     int ns = suggest(&wlst, word, maxSug - 1);
@@ -1416,7 +1423,7 @@ char * SuggestMgr::suggest_morph_for_spelling_error(const char * word)
         free(wlst[maxSug - 1]);
     }
     if (wlst) free(wlst);
-    return p;    
+    return p;
 }
 #endif // END OF HUNSPELL_EXPERIMENTAL CODE
 
@@ -1435,12 +1442,8 @@ int SuggestMgr::ngram(int n, char * s1, const char * s2, int uselen)
     l1 = u8_u16(su1, MAXSWL, s1);
     l2 = u8_u16(su2, MAXSWL, s2);
     if (!l2 || (l1==-1) || (l2==-1)) return 0;
-    // decapitalize dictionary word
-    if (complexprefixes) {
-      mkallsmall_utf(su2+l2-1, 1, pAMgr->get_langnum());
-    } else {
-      mkallsmall_utf(su2, 1, pAMgr->get_langnum());
-    }
+    // lowering dictionary word
+    mkallsmall_utf(su2, l2, pAMgr->get_langnum());
     for (int j = 1; j <= n; j++) {
       ns = 0;
       for (int i = 0; i <= (l1-j); i++) {
@@ -1466,11 +1469,7 @@ int SuggestMgr::ngram(int n, char * s1, const char * s2, int uselen)
     l2 = strlen(s2);
     if (!l2) return 0;
     strcpy(t, s2);
-    if (complexprefixes) {
-      *(t+l2-1) = csconv[((unsigned char)*(t+l2-1))].clower;
-    } else {
     mkallsmall(t, csconv);
-    }
     for (int j = 1; j <= n; j++) {
       ns = 0;
       for (int i = 0; i <= (l1-j); i++) {
@@ -1617,6 +1616,12 @@ void SuggestMgr::lcs(const char * s, const char * s2, int * l1, int * l2, char *
   }
   c = (char *) malloc((m + 1) * (n + 1));
   b = (char *) malloc((m + 1) * (n + 1));
+  if (!c || !b) {
+    if (c) free(c);
+    if (b) free(b);
+    *result = NULL;
+    return;
+  }
   for (i = 1; i <= m; i++) c[i*(n+1)] = 0;
   for (j = 0; j <= n; j++) c[j] = 0;
   for (i = 1; i <= m; i++) {
@@ -1648,6 +1653,7 @@ int SuggestMgr::lcslen(const char * s, const char* s2) {
   char * result;
   int len = 0;
   lcs(s, s2, &m, &n, &result);
+  if (!result) return 0;
   i = m;
   j = n;
   while ((i != 0) && (j != 0)) {
@@ -1659,6 +1665,6 @@ int SuggestMgr::lcslen(const char * s, const char* s2) {
       i--;
     } else j--;
   }
-  if (result) free(result);
+  free(result);
   return len;
 }
