@@ -24,15 +24,23 @@
 //for debugging only
 //#define LOG
 
+#define DEFAULTDICNAME "default"
+
 #ifdef W32
 
 #define gettext
 #define LIBDIR "C:\\Hunspell\\"
-#define HOME "C:\\Hunspell\\"
+#define USEROOODIR "Application Data\\OpenOffice.org 2\\user\\wordbook"
+#define OOODIR "C:\\Program files\\OpenOffice.org 2.3\\share\\dict\\ooo\\;" \
+    "C:\\Program files\\OpenOffice.org 2.2\\share\\dict\\ooo\\;" \
+    "C:\\Program files\\OpenOffice.org 2.1\\share\\dict\\ooo\\;" \
+    "C:\\Program files\\OpenOffice.org 2.0\\share\\dict\\ooo\\"
+#define HOME "%USERPROFILE%\\"
 #define DICBASENAME "hunspell_"
 #define LOGFILE "C:\\Hunspell\\log"
-#define PATHSEP '\\'
-
+#define DIRSEPCH '\\'
+#define DIRSEP "\\"
+#define PATHSEP ";"
 
 #include "textparser.hxx"
 #include "htmlparser.hxx"
@@ -51,17 +59,33 @@
 #include "manparser.hxx"
 #include "firstparser.hxx"
 
-#define LIBDIR "/usr/share/myspell/"
+#define LIBDIR "/usr/share/hunspell:/usr/share/myspell"
+#define USEROOODIR ".openoffice.org2/user/wordbook"
+#define OOODIR "/opt/openoffice.org2.3/share/dict/ooo:" \
+    "/usr/lib/openoffice.org2.3/share/dict/ooo:" \
+    "/opt/openoffice.org2.2/share/dict/ooo:" \
+    "/usr/lib/openoffice.org2.2/share/dict/ooo:" \
+    "/opt/openoffice.org2.1/share/dict/ooo:" \
+    "/usr/lib/openoffice.org2.1/share/dict/ooo:" \
+    "/opt/openoffice.org2.0/share/dict/ooo:" \
+    "/usr/lib/openoffice.org2.0/share/dict/ooo"
 #define HOME getenv("HOME")
 #define DICBASENAME ".hunspell_"
 #define LOGFILE "/tmp/hunspell.log"
-#define PATHSEP '/'
+#define DIRSEPCH '/'
+#define DIRSEP "/"
+#define PATHSEP ":"
+#endif
+
+#ifdef HAVE_ICONV
+#include <iconv.h>
+char text_conv[MAXLNLEN];
 #endif
 
 #if ENABLE_NLS
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
-//#include <langinfo.h>
+#include <langinfo.h>
 #endif
 
 #ifdef HAVE_LIBINTL_H
@@ -107,11 +131,12 @@ struct wordlist {
 // global variables
 
 char * wordchars = NULL;
+char * dicpath = NULL;
 int wordchars_len;
 unsigned short * wordchars_utf16 = NULL;
 int wordchars_utf16_len;
-char * dicname;
-char * privdicname;
+char * dicname = NULL;
+char * privdicname = NULL;
 
 int modified;   // modified file sign
 enum { NORMAL,
@@ -124,15 +149,141 @@ enum { NORMAL,
        AUTO2	    };    // automatic spelling to standard output with sed log
 int filter_mode = NORMAL;
 int printgood = 0; // print only good words and lines
-int unicode = 0;
+int showpath = 0;  // show detected path of the dictionary
+int checkurl = 0;  // check URLs and mail addresses
+char * ui_lang = NULL; // locale for default dic_name
+const char * ui_enc = NULL;  // locale character encoding (default for I/O)
+const char * io_enc = NULL;  // I/O character encoding
+const char * dic_enc = NULL; // dictionary encoding
+char * path = NULL;    // 
 
 // functions
 
-TextParser * newParser(char * wordchars, int format, char * extension, Hunspell * pMS) {
-    TextParser * p = NULL;
-    int utf8 = (strcmp(pMS->get_dic_encoding(), "UTF-8") == 0);
+/* change character encoding */
+char * chenc(char * st, const char * enc1, const char * enc2) {
+    char * out = st;
+#ifdef HAVE_ICONV
+    if (strcmp(enc1, enc2) != 0) {
+	size_t c1 = strlen(st) + 1;
+	size_t c2 = MAXLNLEN;
+	char * source = st;
+	char * dest = text_conv;
+	iconv_t conv = iconv_open(enc2, enc1);
+	size_t res = iconv(conv, (ICONV_CONST char **) &source, &c1, &dest, &c2);
+	iconv_close(conv);
+	if (res != (size_t) -1) out = text_conv;
+    }
+#endif
+    return out;
+}
 
-    if (utf8) {
+TextParser * get_parser(int format, char * extension, Hunspell * pMS) {
+    TextParser * p = NULL;
+    int io_utf8 = 0;
+    dic_enc = pMS->get_dic_encoding();
+#ifdef HAVE_ICONV
+    initialize_utf_tbl(); // also need for 8-bit tokenization
+    if (io_enc) {
+	if ((strcmp(io_enc, "UTF-8") == 0) ||
+	    (strcmp(io_enc, "utf-8") == 0) ||
+	    (strcmp(io_enc, "UTF8") == 0) ||
+	    (strcmp(io_enc, "utf8") == 0)) {
+		io_utf8 = 1;
+		io_enc = "UTF-8";
+	}
+    } else if (ui_enc) {
+	io_enc = ui_enc;
+	if (strcmp(ui_enc, "UTF-8") == 0) io_utf8 = 1;	
+    } else {
+	io_enc = dic_enc;
+	if (strcmp(dic_enc, "UTF-8") == 0) io_utf8 = 1;
+    }
+
+    if (io_utf8) {
+    	wordchars_utf16 = pMS->get_wordchars_utf16(&wordchars_utf16_len);
+	if ((strcmp(dic_enc, "UTF-8") != 0) && pMS->get_wordchars()) {
+	    char * wchars = (char *) pMS->get_wordchars();
+	    int wlen = strlen(wchars);
+	    size_t c1 = wlen;
+	    size_t c2 = MAXLNLEN;
+	    char * dest = text_conv;
+	    iconv_t conv = iconv_open("UTF-8", dic_enc);
+	    iconv(conv, (ICONV_CONST char **) &wchars, &c1, &dest, &c2);
+	    iconv_close(conv);
+	    wordchars_utf16 = (unsigned short *) malloc(sizeof(unsigned short) * wlen);
+	    int n = u8_u16((w_char *) wordchars_utf16, wlen, text_conv);
+	    if (n > 0) flag_qsort(wordchars_utf16, 0, n);
+	    wordchars_utf16_len = n;
+	}
+    } else {
+	// 8-bit input encoding
+	// detect letters by unicodeisalpha() for tokenization
+	char letters[MAXLNLEN];
+	char * pletters = letters;
+	char ch[2];
+	char u8[10];
+	iconv_t conv = iconv_open("UTF-8", io_enc);
+	for (int i = 32; i < 256; i++) {
+	    size_t c1 = 1;
+	    size_t c2 = 10;
+	    char * dest = u8;
+	    u8[0] = '\0';
+	    char * ch8bit = ch;	
+	    ch[0] = (char) i;	    
+	    ch[1] = '\0';
+	    size_t res = iconv(conv, (ICONV_CONST char **) &ch8bit, &c1, &dest, &c2);
+	    if (res != (size_t) -1) {
+		unsigned short idx;
+		w_char w;
+		w.l = 0;
+		w.h = 0;
+		u8_u16(&w, 1, u8);
+		idx = (w.h << 8) + w.l;
+		if (unicodeisalpha(idx)) {
+		    *pletters = (char) i;
+		    pletters++;
+		}
+	    }
+	}
+	*pletters = '\0';
+	iconv_close(conv);
+
+	// UTF-8 wordchars -> 8 bit wordchars
+    	int len = 0;
+    	char * wchars = (char *) pMS->get_wordchars();
+	if (wchars) {
+	    if ((strcmp(dic_enc, "UTF-8")==0)) {
+    		pMS->get_wordchars_utf16(&len);
+	    } else {
+		len = strlen(wchars);
+	    }
+	    char * dest = letters + strlen(letters); // append wordchars
+	    size_t c1 = len + 1;
+	    size_t c2 = len + 1;
+	    iconv_t conv = iconv_open(io_enc, dic_enc);
+	    iconv(conv, (ICONV_CONST char **) &wchars, &c1, &dest, &c2);
+	    iconv_close(conv);		
+	}
+	
+	if (*letters) wordchars = mystrdup(letters);
+    }
+#else
+    if (strcmp(dic_enc, "UTF-8") == 0) {
+        wordchars_utf16 = pMS->get_wordchars_utf16(&wordchars_utf16_len);
+	io_utf8 = 1;
+    } else {
+	char * casechars = get_casechars(dic_enc);
+	wordchars = (char *) pMS->get_wordchars();
+	if (wordchars) {
+	    casechars = (char *) realloc(casechars, strlen(casechars) + strlen(wordchars) + 1);
+	    strcat(casechars, wordchars);
+	}
+	wordchars = casechars;
+    }
+    io_enc = dic_enc;
+#endif
+
+    if (io_utf8) {
         switch (format) {	
         case FMT_LATEX: p = new LaTeXParser(wordchars_utf16, wordchars_utf16_len); break;
         case FMT_HTML: p = new HTMLParser(wordchars_utf16, wordchars_utf16_len); break;
@@ -152,19 +303,19 @@ TextParser * newParser(char * wordchars, int format, char * extension, Hunspell
 	if ((strcmp(extension, "html") == 0) ||
     	    (strcmp(extension, "htm") == 0) ||
 	    (strcmp(extension, "xml") == 0)) {
-                if (utf8) {
+                if (io_utf8) {
                     p = new HTMLParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 		    p = new HTMLParser(wordchars);
                 }
 	} else if (((extension[0] > '0') && (extension[0] <= '9'))) {
-                if (utf8) {
+                if (io_utf8) {
 	            p = new ManParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 	            p = new ManParser(wordchars);
                 }
 	} else if ((strcmp(extension, "tex") == 0)) {
-                if (utf8) {
+                if (io_utf8) {
                     p = new LaTeXParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 	            p = new LaTeXParser(wordchars);
@@ -172,12 +323,13 @@ TextParser * newParser(char * wordchars, int format, char * extension, Hunspell
 	}
     }
     if (!p) {
-        if (utf8) {
+        if (io_utf8) {
             p = new TextParser(wordchars_utf16, wordchars_utf16_len);
         } else {
             p = new TextParser(wordchars);    
         }
     }
+    p->set_url_checking(checkurl);
     return p;
 }
 
@@ -261,6 +413,7 @@ int save_privdic(char * filename, char * filename2, wordlist * w)
 	free(r);
     }
     fclose(dic);
+    return 1;
 }
 
 char * basename(char * s, char c) {
@@ -291,7 +444,7 @@ int bad;
 int lineno = 0;
 int terse_mode = 0;
 
-TextParser * parser = newParser(wordchars, format, NULL, pMS);
+TextParser * parser = get_parser(format, NULL, pMS);
 
 if ((filter_mode == NORMAL) || (filter_mode == PIPE)) {
     if (filter_mode == NORMAL) {
@@ -323,11 +476,12 @@ nextline: while(fgets(buf, MAXLNLEN, fileid)) {
     case '+': {
 		delete parser;
 		parser = new LaTeXParser(wordchars);
+    		parser->set_url_checking(checkurl);
 		break;
 	}
     case '-': {
 		delete parser;
-		parser = newParser(wordchars, format, NULL, pMS);
+		parser = get_parser(format, NULL, pMS);
 		break;
 	}
     case '@': { putdic(buf+1,pMS); break; }
@@ -351,7 +505,7 @@ nextline: while(fgets(buf, MAXLNLEN, fileid)) {
 	buf2 = buf+strlen(buf);
 	if (!privdicname) {
 		strcat(buf,DICBASENAME);
-		strcat(buf,basename(dicname,PATHSEP));
+		strcat(buf,basename(dicname,DIRSEPCH));
 	} else {
 		strcat(buf,privdicname);
 	}
@@ -377,7 +531,7 @@ if (pos >= 0) {
 	    switch (filter_mode) {
 		
 		case BADWORD: {
-			if (! pMS->spell(token)) {
+			if (! pMS->spell(chenc(token, io_enc, dic_enc))) {
 				bad = 1;
 				if (! printgood) fprintf(stdout,"%s\n", token);
 			} else {
@@ -388,7 +542,7 @@ if (pos >= 0) {
 		}
 		
 		case WORDFILTER: {
-			if (! pMS->spell(buf)) {
+			if (! pMS->spell(chenc(buf, io_enc, dic_enc))) {
 				bad = 1;
 				if (! printgood) fprintf(stdout,"%s\n", buf);
 			} else {
@@ -399,7 +553,7 @@ if (pos >= 0) {
 		}
 
 		case BADLINE: {
-			if (! pMS->spell(token)) {
+			if (! pMS->spell(chenc(token, io_enc, dic_enc))) {
 				bad = 1;
 			}
 			free(token);
@@ -410,17 +564,21 @@ if (pos >= 0) {
 		case AUTO:
 		case AUTO2: {
                 FILE * f = (filter_mode == AUTO) ? stderr : stdout;
-		if (! pMS->spell(token)) {
+		if (! pMS->spell(chenc(token, io_enc, dic_enc))) {
 			char ** wlst = NULL;
 			bad = 1;
-			int ns = pMS->suggest_auto(&wlst, token);
+			int ns = pMS->suggest_auto(&wlst, chenc(token, io_enc, dic_enc));
 			if (ns > 0) {
-				parser->change_token(wlst[0]);
+				parser->change_token(chenc(wlst[0], dic_enc, io_enc));
 				if (filter_mode != AUTO2) {
-					fprintf(f,gettext("Line %d: %s -> %s\n"), lineno, token, wlst[0]);
+					fprintf(f,gettext("Line %d: %s -> "), lineno,
+					    chenc(token, io_enc, ui_enc));
+					fprintf(f,gettext("%s\n"),
+					    chenc(wlst[0], dic_enc, ui_enc));
                                         free(wlst[0]);
                                 } else {
-					fprintf(f,"%ds/%s/%s/g; # %s\n", lineno, token, wlst[0], buf);
+					fprintf(f,"%ds/%s/%s/g; # %s\n", lineno,
+					    token, chenc(wlst[0], dic_enc, io_enc), buf);
                                 }
 			}
 			for (int j = 1; j < ns; j++) {
@@ -433,32 +591,62 @@ if (pos >= 0) {
 			continue;			
 		}
 
-		case PIPE:
+		case PIPE: {
+		int info;
+                char * root = NULL;
+		if (pMS->spell(chenc(token, io_enc, dic_enc), &info, &root)) {
+			if (!terse_mode) fprintf(stdout,"*\n");
+			fflush(stdout);
+		} else {
+			char ** wlst = NULL;
+			int ns = pMS->suggest(&wlst, token);
+			if (ns == 0) {
+		    		fprintf(stdout,"# %s %d", token,
+		    		    parser->get_tokenpos() + pos);
+			} else {
+				fprintf(stdout,"& %s %d %d: ", token, ns,
+				    parser->get_tokenpos() + pos);
+				fprintf(stdout,"%s", chenc(wlst[0], dic_enc, io_enc));
+                                free(wlst[0]);
+			}
+			for (int j = 1; j < ns; j++) {
+				fprintf(stdout, ", %s", chenc(wlst[j], dic_enc, io_enc));
+		    		free(wlst[j]);
+			}
+
+			fprintf(stdout, "\n");
+			fflush(stdout);
+			if (wlst) free(wlst);
+		}
+		free(token);
+		continue;
+		}
 		case NORMAL: {
 		int info;
                 char * root = NULL;
-		if (pMS->spell(token, &info, &root)) {
-			if (filter_mode == PIPE) {
-				if (!terse_mode) fprintf(stdout,"*\n");
-			} else if (info & SPELL_COMPOUND) {
+		if (pMS->spell(chenc(token, io_enc, dic_enc), &info, &root)) {
+			if (info & SPELL_COMPOUND) {
 				fprintf(stdout,"-\n");
 			} else if (root) {
-		    		fprintf(stdout,"+ %s\n", root);
+		    		fprintf(stdout,"+ %s\n", chenc(root, dic_enc, ui_enc));
 			} else {
 				fprintf(stdout,"*\n");
 			}			
 			fflush(stdout);
 		} else {
 			char ** wlst = NULL;
-			int ns = pMS->suggest(&wlst, token);
+			int ns = pMS->suggest(&wlst, chenc(token, io_enc, dic_enc));
 			if (ns == 0) {
-		    		fprintf(stdout,"# %s %d", token, parser->get_tokenpos() + pos);
+		    		fprintf(stdout,"# %s %d", chenc(token, io_enc, ui_enc),
+		    		    parser->get_tokenpos() + pos);
 			} else {
-				fprintf(stdout,"& %s %d %d: %s", token, ns, parser->get_tokenpos() + pos, wlst[0]);
+				fprintf(stdout,"& %s %d %d: ", chenc(token, io_enc, ui_enc), ns,
+				    parser->get_tokenpos() + pos);
+				fprintf(stdout,"%s", chenc(wlst[0], dic_enc, ui_enc));
                                 free(wlst[0]);
 			}
 			for (int j = 1; j < ns; j++) {
-				fprintf(stdout, ", %s", wlst[j]);
+				fprintf(stdout, ", %s", chenc(wlst[j], dic_enc, ui_enc));
 		    		free(wlst[j]);
 			}
 
@@ -555,7 +743,7 @@ void dialogscreen(TextParser * parser, char * token,
 	clear();
 
 	if (forbidden) printw(gettext("FORBIDDEN!"));
-	printw(gettext("\t%s\t\tFile: %s\n\n"), token,filename);
+	printw(gettext("\t%s\t\tFile: %s\n\n"), chenc(token, io_enc, ui_enc), filename);
 
 	// handle long lines and tabulators
 
@@ -573,7 +761,7 @@ void dialogscreen(TextParser * parser, char * token,
 	for (int i = 0; i < MAXPREVLINE; i++) {
 		char * line = (char *) calloc(1, x + 1);
 		strncpy(line, lines[prevline] + x * rowindex, x);
-		mvprintw(MAXPREVLINE + 1 - i, 0, "%s", line);
+		mvprintw(MAXPREVLINE + 1 - i, 0, "%s", chenc(line, io_enc, ui_enc));
 		rowindex--;
 		if (rowindex == -1) {
 			prevline++;
@@ -584,18 +772,18 @@ void dialogscreen(TextParser * parser, char * token,
 
 	char * line = (char *) calloc(1, x + 1);
 	strncpy(line, lines[0] + x * rowindex, parser->get_tokenpos() % x);
-	mvprintw(MAXPREVLINE + 1 - beginrow, 0, "%s", line);
+	mvprintw(MAXPREVLINE + 1 - beginrow, 0, "%s", chenc(line, io_enc, ui_enc));
         free(line);
 	attron(A_REVERSE);    
-	printw("%s", token);
+	printw("%s", chenc(token, io_enc, ui_enc));
 	attroff(A_REVERSE);
     
 	mvprintw(MAXPREVLINE + 2, 0, "\n");
 	for (int i = 0; i < ns; i++) {
 		if ((ns > 10) && (i < 10)) {
-			printw(" 0%d: %s\n", i, wlst[i]);
+			printw(" 0%d: %s\n", i, chenc(wlst[i], dic_enc, ui_enc));
 		} else {
-			printw(" %d: %s\n", i, wlst[i]);
+			printw(" %d: %s\n", i, chenc(wlst[i], dic_enc, ui_enc));
 		}
 	}
 
@@ -729,7 +917,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		buf2 = buf+strlen(buf);
 		if (!privdicname) {
 			strcat(buf,DICBASENAME);
-			strcat(buf,basename(dicname,PATHSEP));
+			strcat(buf,basename(dicname,DIRSEPCH));
 		} else {
 			strcat(buf,privdicname);
 		}
@@ -760,7 +948,11 @@ printw(gettext("\n-- Type space to continue -- \n"));
 			*(temp-1) = '\0';
 		} else {
 			char ** poslst = NULL;
+#ifdef HUNSPELL_EXPERIMENTAL
 			int ps = pMS->suggest_pos_stems(&poslst, token);
+#else
+			int ps = 0;
+#endif
 			if (ps > 0) {
 			    strcpy(buf, poslst[0]);
 			    for (int i = 0; i < ps; i++) {
@@ -855,7 +1047,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		    buf2 = buf + strlen(buf);
 		    if (!privdicname) {
 			    strcat(buf,DICBASENAME);
-			    strcat(buf,basename(dicname,PATHSEP));
+			    strcat(buf,basename(dicname,DIRSEPCH));
 		    } else {
 			    strcat(buf,privdicname);
 		    }
@@ -900,10 +1092,10 @@ int interactive_line(TextParser * parser, Hunspell * pMS, char * filename, FILE
 	int dialogexit = 0;
         int info;
 	while ((token=parser->next_token())) {
-		if (! pMS->spell(token, &info, NULL)) {
+		if (! pMS->spell(chenc(token, io_enc, dic_enc), &info, NULL)) {
 			dialogscreen(parser, token, filename, (info & SPELL_FORBIDDEN), NULL, 0); // preview
 			char ** wlst = NULL;
-			int ns = pMS->suggest(&wlst,token);
+			int ns = pMS->suggest(&wlst, chenc(token, io_enc, dic_enc));
 			if (ns==0) {
 				dialogexit = dialog(parser, pMS, token, filename, wlst, ns, (info & SPELL_FORBIDDEN));
 			} else {	    
@@ -936,11 +1128,11 @@ void interactive_interface(Hunspell * pMS, char * filename, int format)
 
     TextParser * parser;
     char * extension = basename(filename, '.');
-    parser = newParser(wordchars, format, extension, pMS);
+    parser = get_parser(format, extension, pMS);
 
     char * tempname = (char *) malloc(strlen(filename) + strlen(TEMPNAME) + 1);
     strcpy(tempname, filename);
-    strcpy(basename(tempname, PATHSEP), TEMPNAME);
+    strcpy(basename(tempname, DIRSEPCH), TEMPNAME);
     
     FILE *tempfile;
 
@@ -985,7 +1177,40 @@ void interactive_interface(Hunspell * pMS, char * filename, int format)
 
 #endif
 #endif
- 
+
+char * add(char * dest, char * st) {
+	if (!dest) {
+	    dest = mystrdup(st);
+	} else {
+	    dest = (char *) realloc(dest, strlen(dest) + strlen(st) + 1);
+	    strcat(dest, st);
+	}
+	return dest;
+}
+
+char * exist2(char * dir, int len, char * name, char * ext) {
+	char buf[MAXLNLEN];
+	const char * sep = (len == 0) ? "": DIRSEP;
+	strncpy(buf, dir, len);
+	strcpy(buf + len, sep);
+	strcat(buf, name);
+	strcat(buf, ext);
+	if (exist(buf)) return mystrdup(buf);
+	return NULL;
+}
+
+// search existing path for file "name + ext"
+char * search(char * begin, char * name, char * ext) {
+	char * end = begin;
+	while (1) {
+	    while (!((*end == *PATHSEP) || (*end == '\0'))) end++;
+	    char * res = exist2(begin, end - begin, name, ext);
+	    if ((*end == '\0') || res) return res;
+	    end++;
+	    begin = end;
+	}
+}
+
 int main(int argc, char** argv)
 {
 	Hunspell * pMS = NULL;
@@ -997,14 +1222,12 @@ int main(int argc, char** argv)
 	char buf2[MAXLNLEN];
 
 	int argstate = 0;
-	dicname = NULL;
-	privdicname = NULL;
 
 #ifdef ENABLE_NLS
 #ifdef HAVE_LOCALE_H
-	setlocale(LC_ALL, "");
+	ui_lang = setlocale(LC_ALL, "");
 	textdomain("hunspell");
-        //if (strcmp(nl_langinfo(CODESET), "UTF-8") == 0) utf8 = 1;
+        ui_enc = nl_langinfo(CODESET);
 #endif
 #endif
 
@@ -1030,42 +1253,50 @@ int main(int argc, char** argv)
 			if (privdicname) free(privdicname);
 			privdicname = mystrdup(argv[i]);
 			argstate = 0;
+		} else if (argstate == 3) {
+			io_enc = argv[i];
+			argstate = 0;
 		} else if (strcmp(argv[i],"-d")==0) argstate=1;
 		else if (strcmp(argv[i],"-p")==0) argstate=2;
+		else if (strcmp(argv[i],"-i")==0) argstate=3;
 		else if ((strcmp(argv[i],"-h") == 0) || (strcmp(argv[i],"--help") == 0)) {
 			fprintf(stderr,gettext("Usage: hunspell [OPTION]... [FILE]...\n"));
 			fprintf(stderr,gettext("Check spelling of each FILE. Without FILE, check standard input.\n"));
 			fprintf(stderr,"\n");
 			fprintf(stderr,gettext("  -1\t\tcheck only first field in lines (delimiter = tabulator)\n"));
 			fprintf(stderr,gettext("  -a\t\tIspell's pipe interface\n"));
+			fprintf(stderr,gettext("  --check-url\tCheck URLs, e-mail addresses and directory paths\n"));
 			fprintf(stderr,gettext("  -d dict\tuse dict dictionary\n"));
+			fprintf(stderr,gettext("  -D\t\tshow detected path of the dictionary\n"));
 			fprintf(stderr,gettext("  -G\t\tprint only correct words or lines\n"));
 			fprintf(stderr,gettext("  -h, --help\tdisplay this help and exit\n"));
 			fprintf(stderr,gettext("  -H\t\tHTML input file format\n"));
+			fprintf(stderr,gettext("  -i enc\tinput encoding\n"));
 			fprintf(stderr,gettext("  -l\t\tprint mispelled words\n"));
 			fprintf(stderr,gettext("  -L\t\tprint lines with mispelled words\n"));
 			fprintf(stderr,gettext("  -n\t\tnroff/troff input file format\n"));
 			fprintf(stderr,gettext("  -p dict\tset dict custom dictionary\n"));
 			fprintf(stderr,gettext("  -t\t\tTeX/LaTeX input file format\n"));
-			fprintf(stderr,gettext("  -u\t\tshow typical misspellings\n"));
-			fprintf(stderr,gettext("  -u2\t\tprint typical misspellings in sed format\n"));
-			fprintf(stderr,gettext("  -U\t\tautomatic correction of typical misspellings to stdout\n"));
+// experimental functions: missing Unicode support
+//			fprintf(stderr,gettext("  -u\t\tshow typical misspellings\n"));
+//			fprintf(stderr,gettext("  -u2\t\tprint typical misspellings in sed format\n"));
+//			fprintf(stderr,gettext("  -U\t\tautomatic correction of typical misspellings to stdout\n"));
 			fprintf(stderr,gettext("  -v, --version\tprint version number\n"));
 			fprintf(stderr,gettext("  -vv\t\tprint Ispell compatible version number\n"));
 			fprintf(stderr,gettext("  -w\t\tprint mispelled words (= lines) from one word/line input.\n"));
 			fprintf(stderr,"\n");
-			fprintf(stderr,gettext("Example: hunspell -d english file.txt  # interactive spelling\n"));
+			fprintf(stderr,gettext("Example: hunspell -d en_US file.txt    # interactive spelling\n"));
 			fprintf(stderr,gettext("         hunspell -l file.txt          # print misspelled words\n"));
-			fprintf(stderr,gettext("         hunspell -u file.txt          # print typical (=serious) misspellings\n"));
+			fprintf(stderr,gettext("         hunspell -i utf8 file.txt     # check UTF-8 encoded file\n"));
 			fprintf(stderr,"\n");
-			fprintf(stderr,gettext("E-mail bug reports to: szoszablya-user@lists.mokk.bme.hu\n"));
+			fprintf(stderr,gettext("Bug reports: http://hunspell.sourceforge.net\n"));
 			exit(0);
 	        } else if ((strcmp(argv[i],"-vv")==0) || (strcmp(argv[i],"-v")==0) || (strcmp(argv[i],"--version")==0)) {
 			fprintf(stdout,gettext(HUNSPELL_PIPE_HEADING));
 			fprintf(stdout,"\n");
                         if (strcmp(argv[i],"-vv")!=0) {
 			    fprintf(stdout,"\n");
-                            fprintf(stdout,gettext("Copyright (C) 2002-2005 Nemeth Laszlo. License: GNU LGPL.\n"));
+                            fprintf(stdout,gettext("Copyright (C) 2002-2007 Nemeth Laszlo. License: GNU LGPL.\n"));
 			    fprintf(stdout,"\n");
 			    fprintf(stdout,gettext("Based on OpenOffice.org's Myspell library.\n"));
 			    fprintf(stdout,gettext("Myspell's copyright (C) Kevin Hendricks, 2001-2002, License: BSD.\n"));
@@ -1099,6 +1330,10 @@ int main(int argc, char** argv)
 			printgood = 1;
 		} else if ((strcmp(argv[i],"-1")==0)) {
 			format = FMT_FIRST;
+		} else if ((strcmp(argv[i],"-D")==0)) {
+			showpath = 1;
+		} else if ((strcmp(argv[i],"--check-url")==0)) {
+			checkurl = 1;
 		} else if ((arg_files==-1) && ((argv[i][0] != '-') && (argv[i][0] != '\0'))) {
 			arg_files = i;
 			if (! exist(argv[i])) { // first check (before time-consuming dic. load)
@@ -1110,30 +1345,39 @@ int main(int argc, char** argv)
 			}
 		}
 	}
-        
+    
         if (printgood && (filter_mode == NORMAL)) filter_mode = BADWORD;
-
+	
 	if (! dicname) {
-		if (! (dicname=getenv("DICTIONARY"))) dicname = mystrdup("default");
+		if (! (dicname=getenv("DICTIONARY"))) {
+			if ((dicname=ui_lang) || (dicname=getenv("LANG"))) {
+			    dicname = mystrdup(dicname);
+			    char * dot = strchr(dicname, '.');
+			    if (dot) *dot = '\0';
+			    char * at = strchr(dicname, '@');
+			    if (at) *at = '\0';
+			} else {
+		            dicname=mystrdup(DEFAULTDICNAME);
+			}
+		} else {
+		    dicname = mystrdup(dicname);
+		}
 	}
+	path = add(mystrdup("."), PATHSEP); // <- check path in local directory
+	path = add(path, PATHSEP);          // <- check path in root directory
+	if (getenv("DICPATH")) path = add(add(path, getenv("DICPATH")), PATHSEP);
+	path = add(add(path, LIBDIR), PATHSEP);
+	path = add(add(add(add(path, HOME), DIRSEP), USEROOODIR), PATHSEP);
+	path = add(path, OOODIR);
 
 	if (!privdicname) privdicname = mystrdup(getenv("WORDLIST"));
 
-        if (strlen(LIBDIR) + strlen(dicname) + 4 < MAXLNLEN) {
-	    sprintf(buf, "%s%s.aff", LIBDIR, dicname);
-	    sprintf(buf2, "%s%s.dic", LIBDIR, dicname);
-        }
-	int libdir = strlen(LIBDIR);
-
-	if (exist(buf + libdir) && exist(buf2 + libdir)) {
-		pMS = new Hunspell(buf + libdir, buf2 + libdir);
-	}
-
-	if ((!pMS) && exist(buf) && exist(buf2)) {
-		pMS = new Hunspell(buf,buf2);
-	}
-
-	if (!pMS) {
+	char * aff = search(path, dicname, ".aff");
+	char * dic = search(path, dicname, ".dic");
+	if (aff && dic) {
+		if (showpath) fprintf(stderr, "%s\n%s\n", aff, dic);
+		pMS = new Hunspell(aff, dic);
+	} else {
 		fprintf(stderr,gettext("Can't open affix or dictionary files.\n"));
 		exit(1);
 	}
@@ -1146,10 +1390,10 @@ int main(int argc, char** argv)
 #endif
 	    if (!privdicname) {
 		strcat(buf,DICBASENAME);
-		strcat(buf,basename(dicname,PATHSEP));
+		strcat(buf,basename(dicname,DIRSEPCH));
 		load_privdic(buf, pMS);
 		strcpy(buf,DICBASENAME);
-		strcat(buf,basename(dicname,PATHSEP));
+		strcat(buf,basename(dicname,DIRSEPCH));
 		load_privdic(buf, pMS);
 	    } else {
 		strcat(buf,privdicname);
@@ -1159,10 +1403,6 @@ int main(int argc, char** argv)
 	    }
         }
 
-	wordchars = mystrdup(pMS->get_wordchars());
-	wordchars_len = strlen(wordchars);
-        wordchars_utf16 = pMS->get_wordchars_utf16(&wordchars_utf16_len);
-
 	if (arg_files==-1) {
 		pipe_interface(pMS, format, stdin);
 	} else {
@@ -1208,7 +1448,13 @@ int main(int argc, char** argv)
 
         if (dicname) free(dicname);
         if (privdicname) free(privdicname);
-	free(wordchars);
+        if (path) free(path);
+        if (aff) free(aff);
+        if (dic) free(dic);
+	if (wordchars) free(wordchars);
+#ifdef HAVE_ICONV
+	free_utf_tbl();
+#endif
 	delete pMS; // not need, because time-consuming
 	return 0;
 }
