@@ -18,8 +18,9 @@ int main(int argc, char **argv)
     FILE *wtclst;
     int i;
     int dp;
-    char buf[101];
+    char buf[10001];
     Hunspell *pMS;
+    char ** s;
 
     /* first parse the command line options */
 
@@ -39,24 +40,20 @@ int main(int argc, char **argv)
     }
 
     pMS = new Hunspell(argv[1], argv[2]);
-    while (fgets(buf, 100, wtclst)) {
+    while (fgets(buf, 10000, wtclst)) {
+      if ((*buf == '@') && (((unsigned char)*(buf+1))) >= 'A') {
         *(buf + strlen(buf) - 1) = '\0';
-        if (*buf == '\0') continue;
-        dp = pMS->spell(buf);
-        fprintf(stdout, "> %s\n", buf);
-	if (dp) {
-            char ** s;
-            int n = pMS->stem(&s, buf);
+        int n = pMS->stem(&s, buf+1);
             if (n) {
-                // stemming is incomplete: search only one stem
-                fprintf(stdout, "%s\n", s[0]);
+                fprintf(stdout, "@%s\n", s[0]);
                 free(s[0]);
                 free(s);
+            } else {
+                fprintf(stdout, "%s\n", buf);
             }
-        } else {
-            fprintf(stdout, "Unknown word.\n");
-            			
-        }
+      } else {
+        fprintf(stdout, "%s", buf);
+      }
     }
     delete pMS;
     return 0;
